#
# Copyright (c) 2020, Grigoriy Kramarenko
# All rights reserved.
# This file is distributed under the same license as the current project.
#
import requests
from copy import deepcopy
from gettext import gettext as _

from astersay.backends.yandex.base import YandexBaseBackend
from astersay.utils import make_voice


class YandexSynthesizer(YandexBaseBackend):
    URL = 'https://tts.api.cloud.yandex.net/speech/v1/tts:synthesize'
    LANGUAGES = ('ru-RU', 'en-US', 'tr-TR')
    VOICES = {
        'ru-RU': ('oksana', 'jane', 'omazh', 'zahar', 'ermil',
                  'alena', 'filipp'),
        'tr-TR': ('silaerkan', 'erkanyavas'),
        'en-US': ('alyss', 'nick'),
    }
    EMOTIONS = ('good', 'evil', 'neutral')
    AUDIO_FORMATS = ('lpcm', 'oggopus')
    HERTZ_LIST = (48000, 16000, 8000)

    def __init__(self, settings, extra=None):
        YandexBaseBackend.__init__(self)
        self.set_token_manager(settings)

        conf = settings.yandex
        self.folder_id = conf['auth']['folder_id']

        synthesizer = deepcopy(conf['synthesizer'])
        if extra:
            synthesizer.update(extra)

        self.storage_path = synthesizer['storage']
        self.convertor = synthesizer['convertor']

        language = synthesizer['lang']
        assert language in self.LANGUAGES
        self.language = language

        voice = synthesizer['voice']
        assert voice in self.VOICES[language]
        self.voice = voice

        emotion = synthesizer['emotion']
        assert emotion in self.EMOTIONS
        self.emotion = emotion

        speed = synthesizer['speed']
        assert 0.1 <= speed <= 3.0
        self.speed = speed

        audio_format = synthesizer['format']
        assert audio_format in self.AUDIO_FORMATS
        self.audio_format = audio_format

        sample_rate_hertz = synthesizer['sample_rate_hertz']
        assert sample_rate_hertz in self.HERTZ_LIST
        self.sample_rate_hertz = sample_rate_hertz

    @property
    def message_params(self):
        return {
            'lang': self.language,
            'voice': self.voice,
            'emotion': self.emotion,
            'speed': self.speed,
            'folderId': self.folder_id,
            'format': self.audio_format,
            'sampleRateHertz': self.sample_rate_hertz,
        }

    def make_message(self, text):
        """
        Создаёт полное сообщение для отправки на сервер и записи в
        спецификацию голосового файла.
        """
        assert len(text) <= 5000
        message = self.message_params
        message['text'] = text
        return message

    def synthesize(self, message):
        """
        Делает запрос сообщения на сервер и возвращает чистые audio-данные
        ответа или None.
        """
        assert isinstance(message, dict)
        self.token_manager.update_token()
        logger_debug = self.logger.debug

        headers = self.token_manager.get_headers()
        logger_debug(_('Заголовки синтеза речи: %s'), headers)
        logger_debug(_('Параметры синтеза речи: %s'), message)
        response = requests.post(self.URL, data=message, headers=headers)
        if response.ok:
            logger_debug(_('Синтез речи выполнен.'))
            # Возвращаем данные полученного файла.
            return response.content
        self.logger.warning(
            _('Ошибка синтеза речи. Сервер вернул код, отличный от 200. %s'),
            response.text,
        )

    def make_voice(self, text, early=False):
        """
        Ищет или создаёт голосовой файл для переданного текста,
        сохраняет его на диск и возвращает в виде пути без расширения,
        пригодного для передачи Астериску.
        """
        return make_voice(
            logger=self.logger,
            convertor=self.convertor,
            synthesize=self.synthesize,
            make_message=self.make_message,
            storage_path=self.storage_path,
            text=text,
            early=early
        )



#
# Copyright (c) 2020, Grigoriy Kramarenko
# All rights reserved.
# This file is distributed under the same license as the current project.
#
import gettext
import os
from astersay import version

VERSION = (0, 23, 0, 'final', 0)


def get_path(*sub):
    base_dir = os.path.dirname(os.path.abspath(__file__))
    if sub:
        return os.path.join(base_dir, *sub)
    return base_dir


def get_version():
    return version.get_version(VERSION, get_path())


__version__ = get_version()

gettext.bindtextdomain(__name__, get_path('locale'))
gettext.textdomain(__name__)



#
# Copyright (c) 2020, Grigoriy Kramarenko
# All rights reserved.
# This file is distributed under the same license as the current project.
#
from astersay.backends.base import BaseBackend
from astersay.backends.yandex.token import YandexTokenManager


class YandexBaseBackend(BaseBackend):
    token_manager = None

    def set_token_manager(self, settings, force=False):
        """
        Устанавливает из любого экземпляра общий для основного класса менеджер
        токена.
        """
        if self.token_manager and not force:
            return self.token_manager
        tm = YandexTokenManager(**settings.yandex['auth'])
        YandexBaseBackend.token_manager = tm
        return tm



#
#  Copyright (c) 2021. Egor Demushin.
#  All rights reserved.
#  This file is distributed under the same license as the current project.
#
# import json
import jwt
import logging
from base64 import b64decode
from gettext import gettext as _
from time import time

from astersay.backends.base import BaseToken, BaseTokenManager
from astersay.exceptions import ImproperlyConfigured

logger = logging.getLogger(__name__)


class TinkoffToken(BaseToken):
    min_length = 32


class TinkoffTokenManager(BaseTokenManager):
    token_class = TinkoffToken

    def __init__(self, private_filename, token_filename, api_key,
                 service_account_id, sub, lifetime=600, **kwargs):
        self.private_filename = private_filename
        self.token_filename = token_filename
        self.api_key = api_key
        self.service_account_id = service_account_id
        self.sub = sub
        self.lifetime = lifetime

        return BaseTokenManager.__init__(self, logger)

    def generate_token(self):
        """
        Запрашивает новый токен у Тинькова.
        """
        if not self.service_account_id:
            raise ImproperlyConfigured(
                _('Для Тинькова не установлен service_account_id.'))
        if not self.api_key:
            raise ImproperlyConfigured(
                _('Для Тинькова не установлен api_key.'))

        with open(self.private_filename, 'r') as private:
            private_key = private.read()
        if not private_key:
            raise ImproperlyConfigured(
                _('Для Тинькова не установлен private_key.'))

        now = int(time())
        payload = {
            "aud": 'tinkoff.cloud.tts tinkoff.cloud.stt',
            "iss": self.service_account_id,
            "iat": now,
            'exp': now + self.lifetime
        }
        token = jwt.encode(
            payload,
            b64decode(private_key),
            algorithm='HS256',
            headers={
                'alg': 'HS256',
                'typ': 'JWT',
                'kid': self.api_key,
            }
        )
        if isinstance(token, bytes):
            token = token.decode('utf-8')

        token = TinkoffToken(token, payload['exp'])
        self.token = token

        logger.debug(_('Новый токен успешно создан: %s'), token)
        return token



#
#  Copyright (c) 2021. Egor Demushin.
#  All rights reserved.
#  This file is distributed under the same license as the current project.
#

import requests
from astersay.utils import import_string


class BaseHttpClient:
    ALLOWED_METHODS = ('get', 'post', 'put', 'path', 'delete')
    auth = None
    parser = None
    method = 'get'
    get_args = None
    post_args = None

    def __init__(self, config):
        assert 'url' in config
        self.url = config['url']
        method = config.get('method')
        if method:
            assert method in self.ALLOWED_METHODS
            self.method = method
        if 'auth' in config:
            self.init_auth(config['auth'])
        if 'parser' in config:
            self.init_parser(config['parser'])
        if 'get_args' in config:
            self.get_args = config['get_args']
        if 'post_args' in config:
            self.post_args = config['post_args']

    def init_auth(self, config):
        auth = import_string(config['class'])
        auth_conf = config.get('settings', {})
        self.auth = auth(**auth_conf)

    def init_parser(self, config: dict):
        parser = import_string(config.get('class'))
        self.parser = parser(config.get('settings'))

    def make_request(self, variables):
        url = self.url.format(**variables)
        params = self.build_request_params(variables)
        response = requests.request(self.method, url, timeout=(2, 5), **params)
        if response.status_code in (200, 201, 203):
            """
            если в настройках не был указан парсер, возвращаем None вместо текста ответа
            """
            result = self.parser.parse(response.text) if self.parser else None
            return True, result
        if response.status_code == 204:
            return True, None
        return False, None

    def build_request_params(self, variables):
        params = {}
        if self.get_args:
            params['params'] = {key: val.format(**variables) for key, val in self.get_args.items()}
        if self.post_args:
            params['data'] = {key: val.format(**variables) for key, val in self.post_args.items()}
        if self.auth:
            key, val = self.auth.get_auth_param()
            if key in params:
                """
                параметры авторизации могут передаваться в GET или POST, тогда их нужно добаить к существующим
                """
                assert isinstance(val, dict)
                params[key] = {**params[key], **val}
            else:
                params[key] = val

        return params



#
# Copyright (c) 2020, Grigoriy Kramarenko
# All rights reserved.
# This file is distributed under the same license as the current project.
#
import json
import jwt
import logging
import requests
from dateutil.parser import parse as parse_datetime
from gettext import gettext as _
from time import time

from astersay.backends.base import BaseToken, BaseTokenManager
from astersay.exceptions import ImproperlyConfigured

logger = logging.getLogger(__name__)


class YandexToken(BaseToken):
    min_length = 128


class YandexTokenManager(BaseTokenManager):
    token_class = YandexToken

    def __init__(self, private_filename, token_filename, key_id,
                 service_account_id, **kwargs):
        self.private_filename = private_filename
        self.token_filename = token_filename
        self.key_id = key_id
        self.service_account_id = service_account_id

        return BaseTokenManager.__init__(self, logger)

    def from_response(cls, jsondata):
        key = jsondata['iamToken']
        expiration = parse_datetime(jsondata['expiresAt']).timestamp()
        return YandexToken(key, expiration)

    def generate_token(self):
        """
        Запрашивает новый IAM-токен у Яндекса.
        """

        # Этот временный токен нужен только на несколько минут, чтобы
        # запросить настоящий токен.
        jwt_token = self.make_json_web_token()

        logger.info(_('Запрашивается новый IAM-токен.'))
        response = requests.post(
            "https://iam.api.cloud.yandex.net/iam/v1/tokens",
            headers={"Content-Type": "application/json"},
            data=json.dumps({'jwt': jwt_token})
        )

        if response.status_code != 200:
            logger.error(_(
                'Ошибка запроса на обновление IAM-токена. '
                'Запрос вернул код, отличный от 200. %s'),
                response.content.decode('utf-8'))
            raise ConnectionError(
                _('Запрос вернул код %s.') % response.status_code)

        token = self.from_response(response.json())
        logger.debug(_('Новый IAM-токен успешно создан: %s'), token)
        return token

    def make_json_web_token(self, lifetime=600):
        """
        Создаёт новый JWT с помощью которого будет периодически запрашиваться
        новый IAM-токен. Время жизни этого токена по умолчанию - 10 минут.
        """
        if not self.service_account_id:
            raise ImproperlyConfigured(
                _('Для Яндекс не установлен service_account_id.'))
        if not self.key_id:
            raise ImproperlyConfigured(
                _('Для Яндекс не установлен key_id.'))

        with open(self.private_filename, 'r') as private:
            private_key = private.read()

        now = int(time())
        payload = {
            'aud': 'https://iam.api.cloud.yandex.net/iam/v1/tokens',
            'iss': self.service_account_id,
            'iat': now,
            'exp': now + lifetime
        }
        token = jwt.encode(
            payload,
            private_key,
            algorithm='PS256',
            headers={
                'kid': self.key_id,
            },
        )
        if isinstance(token, bytes):
            token = token.decode('utf-8')
        logger.info(_('Создан новый JWT-токен: %s...%s'), token[:8], token[-8:])
        return token



# -*- coding: utf-8 -*-
# Generated by the protocol buffer compiler.  DO NOT EDIT!
# source: tinkoff/cloud/longrunning/v1/longrunning.proto
"""Generated protocol buffer code."""
from google.protobuf.internal import enum_type_wrapper
from google.protobuf import descriptor as _descriptor
from google.protobuf import message as _message
from google.protobuf import reflection as _reflection
from google.protobuf import symbol_database as _symbol_database
# @@protoc_insertion_point(imports)

_sym_db = _symbol_database.Default()


from astersay.backends.google.proto.api import annotations_pb2 as google_dot_api_dot_annotations__pb2
from google.protobuf import any_pb2 as google_dot_protobuf_dot_any__pb2
from google.protobuf import duration_pb2 as google_dot_protobuf_dot_duration__pb2
from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from astersay.backends.google.proto.rpc import status_pb2 as google_dot_rpc_dot_status__pb2


DESCRIPTOR = _descriptor.FileDescriptor(
  name='tinkoff/cloud/longrunning/v1/longrunning.proto',
  package='tinkoff.cloud.longrunning.v1',
  syntax='proto3',
  serialized_options=b'ZHstash.tcsbank.ru/stt/tinkoff_cloud_apis/pkg/tinkoff/cloud/longrunning/v1',
  create_key=_descriptor._internal_create_key,
  serialized_pb=b'\n.tinkoff/cloud/longrunning/v1/longrunning.proto\x12\x1ctinkoff.cloud.longrunning.v1\x1a\x1cgoogle/api/annotations.proto\x1a\x19google/protobuf/any.proto\x1a\x1egoogle/protobuf/duration.proto\x1a\x1bgoogle/protobuf/empty.proto\x1a\x17google/rpc/status.proto\"\x81\x02\n\tOperation\x12\n\n\x02id\x18\x01 \x01(\t\x12\r\n\x05group\x18\x02 \x01(\t\x12&\n\x08metadata\x18\x03 \x01(\x0b\x32\x14.google.protobuf.Any\x12;\n\x05state\x18\x04 \x01(\x0e\x32,.tinkoff.cloud.longrunning.v1.OperationState\x12#\n\x05\x65rror\x18\x05 \x01(\x0b\x32\x12.google.rpc.StatusH\x00\x12(\n\x08response\x18\x06 \x01(\x0b\x32\x14.google.protobuf.AnyH\x00\x12\x1b\n\x13x_client_request_id\x18\x07 \x01(\tB\x08\n\x06result\"\xbb\x02\n\x0fOperationFilter\x12\x1a\n\x10\x65xact_service_id\x18\x01 \x01(\tH\x00\x12\x30\n\x0e\x61ny_service_id\x18\x02 \x01(\x0b\x32\x16.google.protobuf.EmptyH\x00\x12\x12\n\x08\x65xact_id\x18\x03 \x01(\tH\x01\x12(\n\x06\x61ny_id\x18\x04 \x01(\x0b\x32\x16.google.protobuf.EmptyH\x01\x12\x15\n\x0b\x65xact_group\x18\x05 \x01(\tH\x02\x12+\n\tany_group\x18\x06 \x01(\x0b\x32\x16.google.protobuf.EmptyH\x02\x12;\n\x05state\x18\x07 \x03(\x0e\x32,.tinkoff.cloud.longrunning.v1.OperationStateB\x0c\n\nservice_idB\x04\n\x02idB\x07\n\x05group\"!\n\x13GetOperationRequest\x12\n\n\x02id\x18\x01 \x01(\t\"N\n\x14WaitOperationRequest\x12\n\n\x02id\x18\x01 \x01(\t\x12*\n\x07timeout\x18\x02 \x01(\x0b\x32\x19.google.protobuf.Duration\"}\n\x15ListOperationsRequest\x12=\n\x06\x66ilter\x18\x01 \x01(\x0b\x32-.tinkoff.cloud.longrunning.v1.OperationFilter\x12\x11\n\tpage_size\x18\x02 \x01(\x05\x12\x12\n\npage_token\x18\x03 \x01(\t\"n\n\x16ListOperationsResponse\x12;\n\noperations\x18\x01 \x03(\x0b\x32\'.tinkoff.cloud.longrunning.v1.Operation\x12\x17\n\x0fnext_page_token\x18\x02 \x01(\t\"W\n\x16\x44\x65leteOperationRequest\x12=\n\x06\x66ilter\x18\x01 \x01(\x0b\x32-.tinkoff.cloud.longrunning.v1.OperationFilter\"W\n\x16\x43\x61ncelOperationRequest\x12=\n\x06\x66ilter\x18\x01 \x01(\x0b\x32-.tinkoff.cloud.longrunning.v1.OperationFilter\"s\n\x16WatchOperationsRequest\x12=\n\x06\x66ilter\x18\x01 \x01(\x0b\x32-.tinkoff.cloud.longrunning.v1.OperationFilter\x12\x1a\n\x12listen_for_updates\x18\x02 \x01(\x08\"U\n\x16OperationsInitialState\x12;\n\noperations\x18\x01 \x03(\x0b\x32\'.tinkoff.cloud.longrunning.v1.Operation\"O\n\x10OperationsUpdate\x12;\n\noperations\x18\x01 \x03(\x0b\x32\'.tinkoff.cloud.longrunning.v1.Operation\"\xe9\x01\n\x17WatchOperationsResponse\x12M\n\rinitial_state\x18\x01 \x01(\x0b\x32\x34.tinkoff.cloud.longrunning.v1.OperationsInitialStateH\x00\x12/\n\rinit_finished\x18\x02 \x01(\x0b\x32\x16.google.protobuf.EmptyH\x00\x12@\n\x06update\x18\x03 \x01(\x0b\x32..tinkoff.cloud.longrunning.v1.OperationsUpdateH\x00\x42\x0c\n\noperations*D\n\x0eOperationState\x12\x0c\n\x08\x45NQUEUED\x10\x00\x12\x0e\n\nPROCESSING\x10\x01\x12\x08\n\x04\x44ONE\x10\x02\x12\n\n\x06\x46\x41ILED\x10\x03\x32\xbf\x06\n\nOperations\x12\x87\x01\n\x0cGetOperation\x12\x31.tinkoff.cloud.longrunning.v1.GetOperationRequest\x1a\'.tinkoff.cloud.longrunning.v1.Operation\"\x1b\x82\xd3\xe4\x93\x02\x15\x12\x13/v1/operations/{id}\x12l\n\rWaitOperation\x12\x32.tinkoff.cloud.longrunning.v1.WaitOperationRequest\x1a\'.tinkoff.cloud.longrunning.v1.Operation\x12\x93\x01\n\x0eListOperations\x12\x33.tinkoff.cloud.longrunning.v1.ListOperationsRequest\x1a\x34.tinkoff.cloud.longrunning.v1.ListOperationsResponse\"\x16\x82\xd3\xe4\x93\x02\x10\x12\x0e/v1/operations\x12\x80\x01\n\x0fWatchOperations\x12\x34.tinkoff.cloud.longrunning.v1.WatchOperationsRequest\x1a\x35.tinkoff.cloud.longrunning.v1.WatchOperationsResponse0\x01\x12\x89\x01\n\x0f\x44\x65leteOperation\x12\x34.tinkoff.cloud.longrunning.v1.DeleteOperationRequest\x1a\x16.google.protobuf.Empty\"(\x82\xd3\xe4\x93\x02\"* /v1/operations/{filter.exact_id}\x12\x93\x01\n\x0f\x43\x61ncelOperation\x12\x34.tinkoff.cloud.longrunning.v1.CancelOperationRequest\x1a\x16.google.protobuf.Empty\"2\x82\xd3\xe4\x93\x02,\"\'/v1/operations/{filter.exact_id}:cancel:\x01*BJZHstash.tcsbank.ru/stt/tinkoff_cloud_apis/pkg/tinkoff/cloud/longrunning/v1b\x06proto3'
  ,
  dependencies=[google_dot_api_dot_annotations__pb2.DESCRIPTOR,google_dot_protobuf_dot_any__pb2.DESCRIPTOR,google_dot_protobuf_dot_duration__pb2.DESCRIPTOR,google_dot_protobuf_dot_empty__pb2.DESCRIPTOR,google_dot_rpc_dot_status__pb2.DESCRIPTOR,])

_OPERATIONSTATE = _descriptor.EnumDescriptor(
  name='OperationState',
  full_name='tinkoff.cloud.longrunning.v1.OperationState',
  filename=None,
  file=DESCRIPTOR,
  create_key=_descriptor._internal_create_key,
  values=[
    _descriptor.EnumValueDescriptor(
      name='ENQUEUED', index=0, number=0,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='PROCESSING', index=1, number=1,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='DONE', index=2, number=2,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
    _descriptor.EnumValueDescriptor(
      name='FAILED', index=3, number=3,
      serialized_options=None,
      type=None,
      create_key=_descriptor._internal_create_key),
  ],
  containing_type=None,
  serialized_options=None,
  serialized_start=1854,
  serialized_end=1922,
)
_sym_db.RegisterEnumDescriptor(_OPERATIONSTATE)

OperationState = enum_type_wrapper.EnumTypeWrapper(_OPERATIONSTATE)
ENQUEUED = 0
PROCESSING = 1
DONE = 2
FAILED = 3



_OPERATION = _descriptor.Descriptor(
  name='Operation',
  full_name='tinkoff.cloud.longrunning.v1.Operation',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='tinkoff.cloud.longrunning.v1.Operation.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='group', full_name='tinkoff.cloud.longrunning.v1.Operation.group', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='metadata', full_name='tinkoff.cloud.longrunning.v1.Operation.metadata', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='state', full_name='tinkoff.cloud.longrunning.v1.Operation.state', index=3,
      number=4, type=14, cpp_type=8, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='error', full_name='tinkoff.cloud.longrunning.v1.Operation.error', index=4,
      number=5, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='response', full_name='tinkoff.cloud.longrunning.v1.Operation.response', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='x_client_request_id', full_name='tinkoff.cloud.longrunning.v1.Operation.x_client_request_id', index=6,
      number=7, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='result', full_name='tinkoff.cloud.longrunning.v1.Operation.result',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=224,
  serialized_end=481,
)


_OPERATIONFILTER = _descriptor.Descriptor(
  name='OperationFilter',
  full_name='tinkoff.cloud.longrunning.v1.OperationFilter',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='exact_service_id', full_name='tinkoff.cloud.longrunning.v1.OperationFilter.exact_service_id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='any_service_id', full_name='tinkoff.cloud.longrunning.v1.OperationFilter.any_service_id', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exact_id', full_name='tinkoff.cloud.longrunning.v1.OperationFilter.exact_id', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='any_id', full_name='tinkoff.cloud.longrunning.v1.OperationFilter.any_id', index=3,
      number=4, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='exact_group', full_name='tinkoff.cloud.longrunning.v1.OperationFilter.exact_group', index=4,
      number=5, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='any_group', full_name='tinkoff.cloud.longrunning.v1.OperationFilter.any_group', index=5,
      number=6, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='state', full_name='tinkoff.cloud.longrunning.v1.OperationFilter.state', index=6,
      number=7, type=14, cpp_type=8, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='service_id', full_name='tinkoff.cloud.longrunning.v1.OperationFilter.service_id',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='id', full_name='tinkoff.cloud.longrunning.v1.OperationFilter.id',
      index=1, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
    _descriptor.OneofDescriptor(
      name='group', full_name='tinkoff.cloud.longrunning.v1.OperationFilter.group',
      index=2, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=484,
  serialized_end=799,
)


_GETOPERATIONREQUEST = _descriptor.Descriptor(
  name='GetOperationRequest',
  full_name='tinkoff.cloud.longrunning.v1.GetOperationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='tinkoff.cloud.longrunning.v1.GetOperationRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=801,
  serialized_end=834,
)


_WAITOPERATIONREQUEST = _descriptor.Descriptor(
  name='WaitOperationRequest',
  full_name='tinkoff.cloud.longrunning.v1.WaitOperationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='id', full_name='tinkoff.cloud.longrunning.v1.WaitOperationRequest.id', index=0,
      number=1, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='timeout', full_name='tinkoff.cloud.longrunning.v1.WaitOperationRequest.timeout', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=836,
  serialized_end=914,
)


_LISTOPERATIONSREQUEST = _descriptor.Descriptor(
  name='ListOperationsRequest',
  full_name='tinkoff.cloud.longrunning.v1.ListOperationsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='tinkoff.cloud.longrunning.v1.ListOperationsRequest.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='page_size', full_name='tinkoff.cloud.longrunning.v1.ListOperationsRequest.page_size', index=1,
      number=2, type=5, cpp_type=1, label=1,
      has_default_value=False, default_value=0,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='page_token', full_name='tinkoff.cloud.longrunning.v1.ListOperationsRequest.page_token', index=2,
      number=3, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=916,
  serialized_end=1041,
)


_LISTOPERATIONSRESPONSE = _descriptor.Descriptor(
  name='ListOperationsResponse',
  full_name='tinkoff.cloud.longrunning.v1.ListOperationsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operations', full_name='tinkoff.cloud.longrunning.v1.ListOperationsResponse.operations', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='next_page_token', full_name='tinkoff.cloud.longrunning.v1.ListOperationsResponse.next_page_token', index=1,
      number=2, type=9, cpp_type=9, label=1,
      has_default_value=False, default_value=b"".decode('utf-8'),
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1043,
  serialized_end=1153,
)


_DELETEOPERATIONREQUEST = _descriptor.Descriptor(
  name='DeleteOperationRequest',
  full_name='tinkoff.cloud.longrunning.v1.DeleteOperationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='tinkoff.cloud.longrunning.v1.DeleteOperationRequest.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1155,
  serialized_end=1242,
)


_CANCELOPERATIONREQUEST = _descriptor.Descriptor(
  name='CancelOperationRequest',
  full_name='tinkoff.cloud.longrunning.v1.CancelOperationRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='tinkoff.cloud.longrunning.v1.CancelOperationRequest.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1244,
  serialized_end=1331,
)


_WATCHOPERATIONSREQUEST = _descriptor.Descriptor(
  name='WatchOperationsRequest',
  full_name='tinkoff.cloud.longrunning.v1.WatchOperationsRequest',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='filter', full_name='tinkoff.cloud.longrunning.v1.WatchOperationsRequest.filter', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='listen_for_updates', full_name='tinkoff.cloud.longrunning.v1.WatchOperationsRequest.listen_for_updates', index=1,
      number=2, type=8, cpp_type=7, label=1,
      has_default_value=False, default_value=False,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1333,
  serialized_end=1448,
)


_OPERATIONSINITIALSTATE = _descriptor.Descriptor(
  name='OperationsInitialState',
  full_name='tinkoff.cloud.longrunning.v1.OperationsInitialState',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operations', full_name='tinkoff.cloud.longrunning.v1.OperationsInitialState.operations', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1450,
  serialized_end=1535,
)


_OPERATIONSUPDATE = _descriptor.Descriptor(
  name='OperationsUpdate',
  full_name='tinkoff.cloud.longrunning.v1.OperationsUpdate',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='operations', full_name='tinkoff.cloud.longrunning.v1.OperationsUpdate.operations', index=0,
      number=1, type=11, cpp_type=10, label=3,
      has_default_value=False, default_value=[],
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
  ],
  serialized_start=1537,
  serialized_end=1616,
)


_WATCHOPERATIONSRESPONSE = _descriptor.Descriptor(
  name='WatchOperationsResponse',
  full_name='tinkoff.cloud.longrunning.v1.WatchOperationsResponse',
  filename=None,
  file=DESCRIPTOR,
  containing_type=None,
  create_key=_descriptor._internal_create_key,
  fields=[
    _descriptor.FieldDescriptor(
      name='initial_state', full_name='tinkoff.cloud.longrunning.v1.WatchOperationsResponse.initial_state', index=0,
      number=1, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='init_finished', full_name='tinkoff.cloud.longrunning.v1.WatchOperationsResponse.init_finished', index=1,
      number=2, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
    _descriptor.FieldDescriptor(
      name='update', full_name='tinkoff.cloud.longrunning.v1.WatchOperationsResponse.update', index=2,
      number=3, type=11, cpp_type=10, label=1,
      has_default_value=False, default_value=None,
      message_type=None, enum_type=None, containing_type=None,
      is_extension=False, extension_scope=None,
      serialized_options=None, file=DESCRIPTOR,  create_key=_descriptor._internal_create_key),
  ],
  extensions=[
  ],
  nested_types=[],
  enum_types=[
  ],
  serialized_options=None,
  is_extendable=False,
  syntax='proto3',
  extension_ranges=[],
  oneofs=[
    _descriptor.OneofDescriptor(
      name='operations', full_name='tinkoff.cloud.longrunning.v1.WatchOperationsResponse.operations',
      index=0, containing_type=None,
      create_key=_descriptor._internal_create_key,
    fields=[]),
  ],
  serialized_start=1619,
  serialized_end=1852,
)

_OPERATION.fields_by_name['metadata'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_OPERATION.fields_by_name['state'].enum_type = _OPERATIONSTATE
_OPERATION.fields_by_name['error'].message_type = google_dot_rpc_dot_status__pb2._STATUS
_OPERATION.fields_by_name['response'].message_type = google_dot_protobuf_dot_any__pb2._ANY
_OPERATION.oneofs_by_name['result'].fields.append(
  _OPERATION.fields_by_name['error'])
_OPERATION.fields_by_name['error'].containing_oneof = _OPERATION.oneofs_by_name['result']
_OPERATION.oneofs_by_name['result'].fields.append(
  _OPERATION.fields_by_name['response'])
_OPERATION.fields_by_name['response'].containing_oneof = _OPERATION.oneofs_by_name['result']
_OPERATIONFILTER.fields_by_name['any_service_id'].message_type = google_dot_protobuf_dot_empty__pb2._EMPTY
_OPERATIONFILTER.fields_by_name['any_id'].message_type = google_dot_protobuf_dot_empty__pb2._EMPTY
_OPERATIONFILTER.fields_by_name['any_group'].message_type = google_dot_protobuf_dot_empty__pb2._EMPTY
_OPERATIONFILTER.fields_by_name['state'].enum_type = _OPERATIONSTATE
_OPERATIONFILTER.oneofs_by_name['service_id'].fields.append(
  _OPERATIONFILTER.fields_by_name['exact_service_id'])
_OPERATIONFILTER.fields_by_name['exact_service_id'].containing_oneof = _OPERATIONFILTER.oneofs_by_name['service_id']
_OPERATIONFILTER.oneofs_by_name['service_id'].fields.append(
  _OPERATIONFILTER.fields_by_name['any_service_id'])
_OPERATIONFILTER.fields_by_name['any_service_id'].containing_oneof = _OPERATIONFILTER.oneofs_by_name['service_id']
_OPERATIONFILTER.oneofs_by_name['id'].fields.append(
  _OPERATIONFILTER.fields_by_name['exact_id'])
_OPERATIONFILTER.fields_by_name['exact_id'].containing_oneof = _OPERATIONFILTER.oneofs_by_name['id']
_OPERATIONFILTER.oneofs_by_name['id'].fields.append(
  _OPERATIONFILTER.fields_by_name['any_id'])
_OPERATIONFILTER.fields_by_name['any_id'].containing_oneof = _OPERATIONFILTER.oneofs_by_name['id']
_OPERATIONFILTER.oneofs_by_name['group'].fields.append(
  _OPERATIONFILTER.fields_by_name['exact_group'])
_OPERATIONFILTER.fields_by_name['exact_group'].containing_oneof = _OPERATIONFILTER.oneofs_by_name['group']
_OPERATIONFILTER.oneofs_by_name['group'].fields.append(
  _OPERATIONFILTER.fields_by_name['any_group'])
_OPERATIONFILTER.fields_by_name['any_group'].containing_oneof = _OPERATIONFILTER.oneofs_by_name['group']
_WAITOPERATIONREQUEST.fields_by_name['timeout'].message_type = google_dot_protobuf_dot_duration__pb2._DURATION
_LISTOPERATIONSREQUEST.fields_by_name['filter'].message_type = _OPERATIONFILTER
_LISTOPERATIONSRESPONSE.fields_by_name['operations'].message_type = _OPERATION
_DELETEOPERATIONREQUEST.fields_by_name['filter'].message_type = _OPERATIONFILTER
_CANCELOPERATIONREQUEST.fields_by_name['filter'].message_type = _OPERATIONFILTER
_WATCHOPERATIONSREQUEST.fields_by_name['filter'].message_type = _OPERATIONFILTER
_OPERATIONSINITIALSTATE.fields_by_name['operations'].message_type = _OPERATION
_OPERATIONSUPDATE.fields_by_name['operations'].message_type = _OPERATION
_WATCHOPERATIONSRESPONSE.fields_by_name['initial_state'].message_type = _OPERATIONSINITIALSTATE
_WATCHOPERATIONSRESPONSE.fields_by_name['init_finished'].message_type = google_dot_protobuf_dot_empty__pb2._EMPTY
_WATCHOPERATIONSRESPONSE.fields_by_name['update'].message_type = _OPERATIONSUPDATE
_WATCHOPERATIONSRESPONSE.oneofs_by_name['operations'].fields.append(
  _WATCHOPERATIONSRESPONSE.fields_by_name['initial_state'])
_WATCHOPERATIONSRESPONSE.fields_by_name['initial_state'].containing_oneof = _WATCHOPERATIONSRESPONSE.oneofs_by_name['operations']
_WATCHOPERATIONSRESPONSE.oneofs_by_name['operations'].fields.append(
  _WATCHOPERATIONSRESPONSE.fields_by_name['init_finished'])
_WATCHOPERATIONSRESPONSE.fields_by_name['init_finished'].containing_oneof = _WATCHOPERATIONSRESPONSE.oneofs_by_name['operations']
_WATCHOPERATIONSRESPONSE.oneofs_by_name['operations'].fields.append(
  _WATCHOPERATIONSRESPONSE.fields_by_name['update'])
_WATCHOPERATIONSRESPONSE.fields_by_name['update'].containing_oneof = _WATCHOPERATIONSRESPONSE.oneofs_by_name['operations']
DESCRIPTOR.message_types_by_name['Operation'] = _OPERATION
DESCRIPTOR.message_types_by_name['OperationFilter'] = _OPERATIONFILTER
DESCRIPTOR.message_types_by_name['GetOperationRequest'] = _GETOPERATIONREQUEST
DESCRIPTOR.message_types_by_name['WaitOperationRequest'] = _WAITOPERATIONREQUEST
DESCRIPTOR.message_types_by_name['ListOperationsRequest'] = _LISTOPERATIONSREQUEST
DESCRIPTOR.message_types_by_name['ListOperationsResponse'] = _LISTOPERATIONSRESPONSE
DESCRIPTOR.message_types_by_name['DeleteOperationRequest'] = _DELETEOPERATIONREQUEST
DESCRIPTOR.message_types_by_name['CancelOperationRequest'] = _CANCELOPERATIONREQUEST
DESCRIPTOR.message_types_by_name['WatchOperationsRequest'] = _WATCHOPERATIONSREQUEST
DESCRIPTOR.message_types_by_name['OperationsInitialState'] = _OPERATIONSINITIALSTATE
DESCRIPTOR.message_types_by_name['OperationsUpdate'] = _OPERATIONSUPDATE
DESCRIPTOR.message_types_by_name['WatchOperationsResponse'] = _WATCHOPERATIONSRESPONSE
DESCRIPTOR.enum_types_by_name['OperationState'] = _OPERATIONSTATE
_sym_db.RegisterFileDescriptor(DESCRIPTOR)

Operation = _reflection.GeneratedProtocolMessageType('Operation', (_message.Message,), {
  'DESCRIPTOR' : _OPERATION,
  '__module__' : 'tinkoff.cloud.longrunning.v1.longrunning_pb2'
  # @@protoc_insertion_point(class_scope:tinkoff.cloud.longrunning.v1.Operation)
  })
_sym_db.RegisterMessage(Operation)

OperationFilter = _reflection.GeneratedProtocolMessageType('OperationFilter', (_message.Message,), {
  'DESCRIPTOR' : _OPERATIONFILTER,
  '__module__' : 'tinkoff.cloud.longrunning.v1.longrunning_pb2'
  # @@protoc_insertion_point(class_scope:tinkoff.cloud.longrunning.v1.OperationFilter)
  })
_sym_db.RegisterMessage(OperationFilter)

GetOperationRequest = _reflection.GeneratedProtocolMessageType('GetOperationRequest', (_message.Message,), {
  'DESCRIPTOR' : _GETOPERATIONREQUEST,
  '__module__' : 'tinkoff.cloud.longrunning.v1.longrunning_pb2'
  # @@protoc_insertion_point(class_scope:tinkoff.cloud.longrunning.v1.GetOperationRequest)
  })
_sym_db.RegisterMessage(GetOperationRequest)

WaitOperationRequest = _reflection.GeneratedProtocolMessageType('WaitOperationRequest', (_message.Message,), {
  'DESCRIPTOR' : _WAITOPERATIONREQUEST,
  '__module__' : 'tinkoff.cloud.longrunning.v1.longrunning_pb2'
  # @@protoc_insertion_point(class_scope:tinkoff.cloud.longrunning.v1.WaitOperationRequest)
  })
_sym_db.RegisterMessage(WaitOperationRequest)

ListOperationsRequest = _reflection.GeneratedProtocolMessageType('ListOperationsRequest', (_message.Message,), {
  'DESCRIPTOR' : _LISTOPERATIONSREQUEST,
  '__module__' : 'tinkoff.cloud.longrunning.v1.longrunning_pb2'
  # @@protoc_insertion_point(class_scope:tinkoff.cloud.longrunning.v1.ListOperationsRequest)
  })
_sym_db.RegisterMessage(ListOperationsRequest)

ListOperationsResponse = _reflection.GeneratedProtocolMessageType('ListOperationsResponse', (_message.Message,), {
  'DESCRIPTOR' : _LISTOPERATIONSRESPONSE,
  '__module__' : 'tinkoff.cloud.longrunning.v1.longrunning_pb2'
  # @@protoc_insertion_point(class_scope:tinkoff.cloud.longrunning.v1.ListOperationsResponse)
  })
_sym_db.RegisterMessage(ListOperationsResponse)

DeleteOperationRequest = _reflection.GeneratedProtocolMessageType('DeleteOperationRequest', (_message.Message,), {
  'DESCRIPTOR' : _DELETEOPERATIONREQUEST,
  '__module__' : 'tinkoff.cloud.longrunning.v1.longrunning_pb2'
  # @@protoc_insertion_point(class_scope:tinkoff.cloud.longrunning.v1.DeleteOperationRequest)
  })
_sym_db.RegisterMessage(DeleteOperationRequest)

CancelOperationRequest = _reflection.GeneratedProtocolMessageType('CancelOperationRequest', (_message.Message,), {
  'DESCRIPTOR' : _CANCELOPERATIONREQUEST,
  '__module__' : 'tinkoff.cloud.longrunning.v1.longrunning_pb2'
  # @@protoc_insertion_point(class_scope:tinkoff.cloud.longrunning.v1.CancelOperationRequest)
  })
_sym_db.RegisterMessage(CancelOperationRequest)

WatchOperationsRequest = _reflection.GeneratedProtocolMessageType('WatchOperationsRequest', (_message.Message,), {
  'DESCRIPTOR' : _WATCHOPERATIONSREQUEST,
  '__module__' : 'tinkoff.cloud.longrunning.v1.longrunning_pb2'
  # @@protoc_insertion_point(class_scope:tinkoff.cloud.longrunning.v1.WatchOperationsRequest)
  })
_sym_db.RegisterMessage(WatchOperationsRequest)

OperationsInitialState = _reflection.GeneratedProtocolMessageType('OperationsInitialState', (_message.Message,), {
  'DESCRIPTOR' : _OPERATIONSINITIALSTATE,
  '__module__' : 'tinkoff.cloud.longrunning.v1.longrunning_pb2'
  # @@protoc_insertion_point(class_scope:tinkoff.cloud.longrunning.v1.OperationsInitialState)
  })
_sym_db.RegisterMessage(OperationsInitialState)

OperationsUpdate = _reflection.GeneratedProtocolMessageType('OperationsUpdate', (_message.Message,), {
  'DESCRIPTOR' : _OPERATIONSUPDATE,
  '__module__' : 'tinkoff.cloud.longrunning.v1.longrunning_pb2'
  # @@protoc_insertion_point(class_scope:tinkoff.cloud.longrunning.v1.OperationsUpdate)
  })
_sym_db.RegisterMessage(OperationsUpdate)

WatchOperationsResponse = _reflection.GeneratedProtocolMessageType('WatchOperationsResponse', (_message.Message,), {
  'DESCRIPTOR' : _WATCHOPERATIONSRESPONSE,
  '__module__' : 'tinkoff.cloud.longrunning.v1.longrunning_pb2'
  # @@protoc_insertion_point(class_scope:tinkoff.cloud.longrunning.v1.WatchOperationsResponse)
  })
_sym_db.RegisterMessage(WatchOperationsResponse)


DESCRIPTOR._options = None

_OPERATIONS = _descriptor.ServiceDescriptor(
  name='Operations',
  full_name='tinkoff.cloud.longrunning.v1.Operations',
  file=DESCRIPTOR,
  index=0,
  serialized_options=None,
  create_key=_descriptor._internal_create_key,
  serialized_start=1925,
  serialized_end=2756,
  methods=[
  _descriptor.MethodDescriptor(
    name='GetOperation',
    full_name='tinkoff.cloud.longrunning.v1.Operations.GetOperation',
    index=0,
    containing_service=None,
    input_type=_GETOPERATIONREQUEST,
    output_type=_OPERATION,
    serialized_options=b'\202\323\344\223\002\025\022\023/v1/operations/{id}',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='WaitOperation',
    full_name='tinkoff.cloud.longrunning.v1.Operations.WaitOperation',
    index=1,
    containing_service=None,
    input_type=_WAITOPERATIONREQUEST,
    output_type=_OPERATION,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='ListOperations',
    full_name='tinkoff.cloud.longrunning.v1.Operations.ListOperations',
    index=2,
    containing_service=None,
    input_type=_LISTOPERATIONSREQUEST,
    output_type=_LISTOPERATIONSRESPONSE,
    serialized_options=b'\202\323\344\223\002\020\022\016/v1/operations',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='WatchOperations',
    full_name='tinkoff.cloud.longrunning.v1.Operations.WatchOperations',
    index=3,
    containing_service=None,
    input_type=_WATCHOPERATIONSREQUEST,
    output_type=_WATCHOPERATIONSRESPONSE,
    serialized_options=None,
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='DeleteOperation',
    full_name='tinkoff.cloud.longrunning.v1.Operations.DeleteOperation',
    index=4,
    containing_service=None,
    input_type=_DELETEOPERATIONREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=b'\202\323\344\223\002\"* /v1/operations/{filter.exact_id}',
    create_key=_descriptor._internal_create_key,
  ),
  _descriptor.MethodDescriptor(
    name='CancelOperation',
    full_name='tinkoff.cloud.longrunning.v1.Operations.CancelOperation',
    index=5,
    containing_service=None,
    input_type=_CANCELOPERATIONREQUEST,
    output_type=google_dot_protobuf_dot_empty__pb2._EMPTY,
    serialized_options=b'\202\323\344\223\002,\"\'/v1/operations/{filter.exact_id}:cancel:\001*',
    create_key=_descriptor._internal_create_key,
  ),
])
_sym_db.RegisterServiceDescriptor(_OPERATIONS)

DESCRIPTOR.services_by_name['Operations'] = _OPERATIONS

# @@protoc_insertion_point(module_scope)



#
#  Copyright (c) 2021. Egor Demushin.
#  All rights reserved.
#  This file is distributed under the same license as the current project.
#
import grpc
import threading
from copy import deepcopy
from gettext import gettext as _

from astersay.backends.base import Listener
from astersay.backends.tinkoff.base import TinkoffBaseBackend
from astersay.backends.tinkoff.proto import stt_pb2, stt_pb2_grpc


class TinkoffRecognizer(TinkoffBaseBackend):
    pause = True
    HOST = 'stt.tinkoff.ru:443'
    LANGUAGES = ('ru-RU',)
    AUDIO_ENCODINGS = ('LINEAR16', 'MULAW', 'ALAW', 'RAW_OPUS', 'MPEG_AUDIO')
    HERTZ_LIST = (8000, 16000, 32000, 48000)
    # Счётчик текущих предварительных результатов. Сбрасывается финальным.
    partials_counter = 0

    def __init__(self, settings, extra=None):
        TinkoffBaseBackend.__init__(self)
        self.set_token_manager(settings)

        conf = settings.tinkoff

        recognizer = deepcopy(conf['recognizer'])
        if extra:
            recognizer.update(extra)

        language = recognizer['language_code']
        assert language in self.LANGUAGES
        self.language = language

        self.profanity_filter = bool(recognizer['profanity_filter'])
        self.enable_automatic_punctuation = bool(recognizer['enable_automatic_punctuation'])

        audio_encoding = recognizer['audio_encoding']
        assert audio_encoding in self.AUDIO_ENCODINGS
        self.audio_encoding = getattr(stt_pb2.AudioEncoding, audio_encoding)

        sample_rate_hertz = recognizer['sample_rate_hertz']
        assert sample_rate_hertz in self.HERTZ_LIST
        self.sample_rate_hertz = sample_rate_hertz

        chunk_size = recognizer['chunk_size']
        assert 1024 <= chunk_size <= 4000 and chunk_size % 2 == 0
        self.chunk_size = chunk_size

        self.enable_interim_results = recognizer['enable_interim_results']
        self.interim_results_interval = recognizer['interim_results_interval']

        self._credentials = grpc.ssl_channel_credentials()

    def _build_first_request(self):
        request = stt_pb2.StreamingRecognizeRequest()
        config = request.streaming_config.config
        config.encoding = self.audio_encoding
        config.sample_rate_hertz = self.sample_rate_hertz
        config.num_channels = 1
        config.language_code = self.language
        config.profanity_filter = self.profanity_filter
        config.enable_automatic_punctuation = self.enable_automatic_punctuation
        interim_results_config = request.streaming_config.interim_results_config
        interim_results_config.enable_interim_results = self.enable_interim_results
        interim_results_config.interval = self.interim_results_interval
        return request

    def _generate_requests(self, listener):
        yield self._build_first_request()
        for chunk in listener.listen():
            if not self.pause:
                request = stt_pb2.StreamingRecognizeRequest()
                request.audio_content = chunk
                yield request

    def _recognize(self, listener):
        """Возвращает итератор ответов Тинькова."""
        self.token_manager.update_token()

        metadata = self.token_manager.get_metadata()
        try:
            stub = stt_pb2_grpc.SpeechToTextStub(
                grpc.secure_channel(self.HOST, self._credentials))
            responses = stub.StreamingRecognize(
                self._generate_requests(listener), metadata=metadata)
            for response in responses:
                for result in response.results:
                    yield result
        except grpc._channel._MultiThreadedRendezvous as e:
            self.logger.warning(
                'Stop iteration by _MultiThreadedRendezvous.code=%s.',
                e._state.code)
        except Exception as e:
            self.logger.critical(str(e), exc_info=e)

    def listen(self, dialog):
        """
        Распознавание текста для диалога производится вплоть до его завершения
        или превышения объёма стрима. Возвращает итератор текста.
        """
        logger_debug = self.logger.debug
        logger_info = self.logger.info
        logger_info(_('Начинается распознавание речи для диалога.'))

        listener = Listener(dialog.stream, dialog.amplitude)
        self.listener = listener
        # Максимальный размер потокового файла для Тинькова не ограничен.
        # listener.max_size = 10485760
        listener.chunk_size = self.chunk_size

        text_to_text_buffer = dialog.text_to_text_buffer
        get_text_buffer = dialog.get_last_text_buffer

        # Максимальное время потока для Тинькова не ограничено.
        # Но мы ограничимся 60 минутами.
        timer = threading.Timer(3600, listener.stop)
        timer.start()
        self.partials_counter = 0
        text_buffer = None
        for answer in self._recognize(listener):
            # Каждый раз получаем самый последний буфер, т.к. он может быть
            # создан в процессе работы. Но для сохранения финальных результатов
            # используем запомненный.
            if text_buffer is None:
                text_buffer = get_text_buffer()
                logger_info(_('Работаем с буфером %s'), text_buffer.key)
            chunk = answer.recognition_result
            alternatives = chunk.alternatives
            text = alternatives[0].transcript
            logger_debug(_('Ответ Тинькова: %s'), text)
            if answer.is_final:
                logger_info(
                    _('Финальный текст в буфер %(buffer)s: %(text)s'),
                    {'text': text, 'buffer': text_buffer.key}
                )
                text_to_text_buffer(text, fixed=True, text_buffer=text_buffer)
                self.partials_counter = 0
                # После каждого финального текста сбрасываем привязку к буферу.
                text_buffer = None
            else:
                logger_debug(
                    _('Предварительный текст в буфер %(buffer)s: %(text)s'),
                    {'text': text, 'buffer': text_buffer.key}
                )
                alt = '\n'.join([a.transcript for a in alternatives if a.transcript])
                text_to_text_buffer(alt, fixed=False, text_buffer=text_buffer)
                self.partials_counter += 1
            if dialog.is_closed:
                listener.stop()
                logger_debug(_('Диалог прекратился и слушатель тоже.'))
                break
        logger_debug(_('Цикл завершён.'))
        timer.cancel()
        dialog.stream_size += listener.total_size



#!/usr/bin/env python3
"""
Пример плагина для Astresay. Первым аргументом для него должен выступать
путь к временному JSON-файлу с данными, полученными в процессе работы
диалога.
Плагины необходимы для переопределения или создания новых переменных, а также
для обращения к другим, например, сетевым ресурсам.
"""
import json
import sys

# Считываем файл с данными для плагина.
filename = sys.argv[1]
with open(filename) as f:
    plugin_data = json.load(f)

#######################
# Начало вашего кода. #
#######################

# Словарь plugin_data содержит:
# 'dialog_id' - UUID диалога с виде строки с дефисами,
# 'agi_params' - все переменные переданные в EAGI,
# 'work_dir' - путь к рабочему каталогу,
# 'speech' - распознанная речь текущего блок-скрипта,
# 'result': результат работы текущего блок-скрипта,
# 'variables' - словарь всех переменных определённых ранее,
# 'chain': список движения блок-скриптов с их результатами и речью,
# 'text_buffers': перечень текстовых буферов определённых ранее,
# 'config': словарь дополнительных параметров для плагина,

print('Получены данные для плагина.')
print('dialog_id:', plugin_data['dialog_id'])
print('agi_params:', plugin_data['agi_params'])
print('work_dir:', plugin_data['work_dir'])
print('speech:', plugin_data['speech'])
print('result:', plugin_data['result'])
print('variables:', plugin_data['variables'])
print('chain:', plugin_data['chain'])
print('text_buffers:', plugin_data['text_buffers'])
print('config:', plugin_data['config'])

# Производим операции с ними. Например, добавляем новую переменную:
variables = plugin_data['variables']
variables['example_plugin_var'] = 'Плагин определил свою переменную.'

# Для вывода в логи задайте сообщения списками с определёнными именами:
plugin_data['log_debug'] = [
    'Example plugin DEBUG string 1',
    'Example plugin DEBUG string 2',
]
plugin_data['log_info'] = [
    'Example plugin INFO string 1',
    'Example plugin INFO string 2',
]
plugin_data['log_warning'] = [
    'Example plugin WARNING string 1',
    'Example plugin WARNING string 2',
]
plugin_data['log_error'] = [
    'Example plugin DEBUG string 1',
    'Example plugin DEBUG string 2',
]
# Для версии 0.16 именами были: "debug", "info", "warnings" и "errors".
# Они будут вызывать "warning" с версии 0.20 и будут окончательно убраны
# в версии 0.30. Не используйте их.

# Для плагинов "plugin_before" и "plugin_after" блок-скрипта "simple_say"
# Вы можете переопределить результат выполнения блок скрипта, чтобы пойти по
# цепочке `fail`, например так:
plugin_data['result'] = False
# Для других типов блок скриптов это переопределение никак не повлияет.


######################
# Конец вашего кода. #
######################

# Перезаписываем файл с данными для возврата в диалог.
# Вы можете закомментировать этот код, если ничего не нужно сообщать диалогу,
# ни логов, ни переменных, ничего...
with open(filename, 'w') as f:
    json.dump(plugin_data, f)

# Обязательно указываем, что плагин успешно завершён, чтобы не прервать диалог.
sys.exit()



#!/usr/bin/env python3
"""
Пример текстового процессора для Astresay. Аргументом для него
выступает текст, полученный в процессе работы диалога.

Тестирование:

    ./are_you_from.py "я живу в комсомольске на амуре"
    ./are_you_from.py "комсомольск-на-амуре"

Заметьте, что текстовые процессоры отличаются от плагинов:

    1. Процессоры вызываются на каждую итерацию поиска распознанного текста,
       т.е. примерно раз в полсекунды и это зависит от настроек блок-скрипта.
       А плагины вызываются 1 раз при использовании в блок-скрипте.
    2. Процессоры оперируют только переданным текстом, а плагины - всеми
       параметрами и доступными переменными.

"""

import sys

try:
    # Предпочтительнее использовать эту библиотеку, она на C.
    from Levenshtein import distance
except ImportError:
    from astersay.utils import levenshtein_distance as distance


# Варианты названий населённых пунктов, их минимальныая дистанция нахождения
# и настоящее название, возвращаемое в диалог.
NAMES = {
    'москва': (3, 'Москва'),
    'санкт': (3, 'Санкт-Петербург'),
    'санкт петербург': (3, 'Санкт-Петербург'),
    'петербург': (3, 'Санкт-Петербург'),
    'питер': (2, 'Санкт-Петербург'),
    'ленинград': (3, 'Санкт-Петербург'),
    'новосибирск': (3, 'Новосибирск'),
    'екатеринбург': (4, 'Екатеринбург'),
    'екатерин': (3, 'Екатеринбург'),
    'омск': (1, 'Омск'),
    'томск': (1, 'Томск'),
    'тула': (1, 'Тула'),
    'ярославль': (3, 'Ярославль'),
    'ростов на дону': (4, 'Ростов-на-Дону'),
    'ростов': (3, 'Ростов'),
    'ростов': (3, 'Ростов'),
    'артём': (3, 'Артём'),
    'артем': (3, 'Артём'),
    'владивосток': (3, 'Владивосток'),
    'хабаровск': (3, 'Хабаровск'),
    'комсомольск на амуре': (5, 'Комсомольск-на-Амуре'),
    'комсомольск': (3, 'Комсомольск-на-Амуре'),
    'биробиджан': (3, 'Биробиджан'),
    'амгунь': (3, 'Амгунь'),
    'чугуевка': (3, 'посёлок Чугуевка'),
    'чугуев': (3, 'посёлок Чугуевка'),
}


def send_result(key):
    """
    Выводим на стандартный вывод текст, чтобы передать его обратно в диалог.
    """
    if key in NAMES:
        value = NAMES[key][1]
    else:
        value = ''
    print(value)
    return value


def get_text_index(text):
    """Возвращает минимальный индекс дистанции по всему тексту."""
    index = []
    for key, value in NAMES.items():
        d = distance(text, key)
        if value[0] >= d:
            index.append((d, key))
    index.sort()
    return index[0] if index else None


def get_word_index(text):
    """Возвращает минимальный индекс дистанции по словам."""
    words = [x for x in text.split(' ') if x]
    index = []
    for key, value in NAMES.items():
        minimum = value[0]
        for word in words:
            d = distance(word, key)
            if minimum >= d:
                index.append((d, key))
    index.sort()
    return index[0] if index else None


def process(text):
    """
    Производит всю необходимую обработку текста и отправляет найденное
    в диалог.
    """
    text_index = get_text_index(text)
    # Когда текст полностью подходит с нулевой дистанцией.
    if text_index and text_index[0] == 0:
        return send_result(text_index[1])

    word_index = get_word_index(text)
    # Когда слово полностью подходит с нулевой дистанцией.
    if word_index and word_index[0] == 0:
        return send_result(word_index[1])

    # Выбираем наименьшее.
    if text_index and word_index:
        if text_index[0] <= word_index[0]:
            return send_result(text_index[1])
        return send_result(word_index[1])
    elif text_index:
        return send_result(text_index[1])
    elif word_index:
        return send_result(word_index[1])

    return send_result('')


# Текст из буфера диалога приходит одним параметром, слова разделены пробелами.
process(sys.argv[1].lower())
# Обязательно указываем, что процессор успешно завершён, чтобы не прервать диалог.
sys.exit()



#
# Copyright (c) 2020, Grigoriy Kramarenko
# All rights reserved.
# This file is distributed under the same license as the current project.
#
import grpc
import threading
from copy import deepcopy
from gettext import gettext as _

from astersay.backends.base import Listener
from astersay.backends.yandex.base import YandexBaseBackend
# from yandex.cloud.ai.stt.v2.stt_service_pb2 (
#     RecognitionSpec, RecognitionConfig, StreamingRecognitionReques
# )
# from yandex.cloud.ai.stt.v2.stt_service_pb2_grpc import SttServiceStub
from astersay.backends.yandex.proto.stt_service_pb2 import (
    RecognitionSpec, RecognitionConfig, StreamingRecognitionRequest,
)
from astersay.backends.yandex.proto.stt_service_pb2_grpc import SttServiceStub


class YandexRecognizer(YandexBaseBackend):
    pause = True
    HOST = 'stt.api.cloud.yandex.net:443'
    LANGUAGES = ('ru-RU', 'en-US', 'tr-TR')
    MODELS = {
        'ru-RU': ('general', 'general:rc'),
        'tr-TR': ('general', 'maps'),
        'en-US': ('general', 'maps'),
    }
    AUDIO_ENCODINGS = ('LINEAR16_PCM', 'OGG_OPUS')
    HERTZ_LIST = (48000, 16000, 8000)
    # Счётчик текущих предварительных результатов. Сбрасывается финальным.
    partials_counter = 0

    def __init__(self, settings, extra=None):
        YandexBaseBackend.__init__(self)
        self.set_token_manager(settings)

        conf = settings.yandex
        self.folder_id = conf['auth']['folder_id']

        recognizer = deepcopy(conf['recognizer'])
        if extra:
            recognizer.update(extra)

        language = recognizer['language_code']
        assert language in self.LANGUAGES
        self.language = language

        model = recognizer['model']
        assert model in self.MODELS[language]
        self.model = model

        self.profanity_filter = bool(recognizer['profanity_filter'])
        self.partial_results = bool(recognizer['partial_results'])
        self.single_utterance = bool(recognizer['single_utterance'])
        self.raw_results = bool(recognizer['raw_results'])

        audio_encoding = recognizer['audio_encoding']
        assert audio_encoding in self.AUDIO_ENCODINGS
        self.audio_encoding = audio_encoding

        sample_rate_hertz = recognizer['sample_rate_hertz']
        assert sample_rate_hertz in self.HERTZ_LIST
        self.sample_rate_hertz = sample_rate_hertz

        chunk_size = recognizer['chunk_size']
        assert 1024 <= chunk_size <= 4000 and chunk_size % 2 == 0
        self.chunk_size = chunk_size

        self._credentials = grpc.ssl_channel_credentials()

    def _recognize(self, listener):
        """Возвращает итератор ответов Яндекса."""
        self.token_manager.update_token()

        # Задаём настройки распознавания.
        specification = RecognitionSpec(
            language_code=self.language,
            model=self.model,
            profanity_filter=self.profanity_filter,
            partial_results=self.partial_results,
            single_utterance=self.single_utterance,
            # raw_results=self.raw_results,  # TODO: Не работает, починить.
            audio_encoding=self.audio_encoding,
            sample_rate_hertz=self.sample_rate_hertz
        )
        streaming_config = RecognitionConfig(
            specification=specification, folder_id=self.folder_id)

        def get_request_iter():
            # Отправляется сообщение с настройками распознавания.
            yield StreamingRecognitionRequest(config=streaming_config)
            for chunk in listener.listen():
                if self.pause:
                    chunk = b''
                yield StreamingRecognitionRequest(audio_content=chunk)

        # Установить соединение с сервером.
        channel = grpc.secure_channel(self.HOST, self._credentials)
        stub = SttServiceStub(channel)
        metadata = self.token_manager.get_metadata()
        iter = stub.StreamingRecognize(get_request_iter(), metadata=metadata)
        try:
            for value in iter:
                yield value
        except grpc._channel._MultiThreadedRendezvous as e:
            self.logger.warning(
                'Stop iteration by _MultiThreadedRendezvous.code=%s.',
                e._state.code)
        except Exception as e:
            self.logger.critical(str(e), exc_info=e)

    def listen(self, dialog):
        """
        Распознавание текста для диалога производится вплоть до его завершения
        или превышения объёма стрима. Возвращает итератор текста.
        """
        logger_debug = self.logger.debug
        logger_info = self.logger.info
        logger_info(_('Начинается распознавание речи для диалога.'))

        listener = Listener(dialog.stream, dialog.amplitude)
        self.listener = listener
        # Максимальный размер потокового файла для Яндекса - 10 мегабайт.
        listener.max_size = 10485760
        listener.chunk_size = self.chunk_size

        text_to_text_buffer = dialog.text_to_text_buffer
        get_text_buffer = dialog.get_last_text_buffer

        # Максимальное время потока для Яндекса - 5 минут.
        timer = threading.Timer(300, listener.stop)
        timer.start()
        self.partials_counter = 0
        text_buffer = None
        for answer in self._recognize(listener):
            # Каждый раз получаем самый последний буфер, т.к. он может быть
            # создан в процессе работы. Но для сохранения финальных результатов
            # используем запомненный.
            if text_buffer is None:
                text_buffer = get_text_buffer()
                logger_info(_('Работаем с буфером %s'), text_buffer.key)
            chunk = answer.chunks[0]
            alternatives = chunk.alternatives
            text = alternatives[0].text
            logger_debug(_('Ответ Яндекса: %s'), text)
            if chunk.final:
                logger_info(
                    _('Финальный текст в буфер %(buffer)s: %(text)s'),
                    {'text': text, 'buffer': text_buffer.key}
                )
                text_to_text_buffer(text, fixed=True, text_buffer=text_buffer)
                self.partials_counter = 0
                # После каждого финального текста сбрасываем привязку к буферу.
                text_buffer = None
            else:
                logger_debug(
                    _('Предварительный текст в буфер %(buffer)s: %(text)s'),
                    {'text': text, 'buffer': text_buffer.key}
                )
                alt = '\n'.join([a.text for a in alternatives if a.text])
                text_to_text_buffer(alt, fixed=False, text_buffer=text_buffer)
                self.partials_counter += 1
            if dialog.is_closed:
                listener.stop()
                logger_debug(_('Диалог прекратился и слушатель тоже.'))
                break
        logger_debug(_('Цикл завершён.'))
        timer.cancel()
        dialog.stream_size += listener.total_size



#!/usr/bin/env python3
import json
import sys
from astersay.plugins.general_http_plugin import BaseHttpClient

# Считываем файл с данными для плагина.
filename = sys.argv[1]
with open(filename) as f:
    plugin_data = json.load(f)

result, data = False, {}
try:
    requester = BaseHttpClient(plugin_data['config'])
    result, data = requester.make_request(plugin_data['variables'])
except Exception as ex:
    plugin_data['log_error'] = [
        f"Ошибка при работе базового http плагина: {str(ex)}"
    ]

variables = plugin_data['variables']
variables.update(data)

plugin_data['result'] = result

print('variables:', plugin_data['variables'])
# Для других типов блок скриптов это переопределение никак не повлияет.


######################
# Конец вашего кода. #
######################

# Перезаписываем файл с данными для возврата в диалог.
# Вы можете закомментировать этот код, если ничего не нужно сообщать диалогу,
# ни логов, ни переменных, ничего...
with open(filename, 'w') as f:
    json.dump(plugin_data, f)

# Обязательно указываем, что плагин успешно завершён, чтобы не прервать диалог.
sys.exit()



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from astersay.backends.tinkoff.proto import longrunning_pb2 as tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2
from astersay.backends.tinkoff.proto import stt_pb2 as tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2


class SpeechToTextStub(object):
    """Speech recognition.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.Recognize = channel.unary_unary(
                '/tinkoff.cloud.stt.v1.SpeechToText/Recognize',
                request_serializer=tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.RecognizeRequest.SerializeToString,
                response_deserializer=tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.RecognizeResponse.FromString,
                )
        self.StreamingRecognize = channel.stream_stream(
                '/tinkoff.cloud.stt.v1.SpeechToText/StreamingRecognize',
                request_serializer=tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.StreamingRecognizeRequest.SerializeToString,
                response_deserializer=tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.StreamingRecognizeResponse.FromString,
                )
        self.LongRunningRecognize = channel.unary_unary(
                '/tinkoff.cloud.stt.v1.SpeechToText/LongRunningRecognize',
                request_serializer=tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.LongRunningRecognizeRequest.SerializeToString,
                response_deserializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.Operation.FromString,
                )


class SpeechToTextServicer(object):
    """Speech recognition.
    """

    def Recognize(self, request, context):
        """Method to recognize whole audio at once: sending complete audio, getting complete recognition result.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamingRecognize(self, request_iterator, context):
        """Method for streaming recognition.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def LongRunningRecognize(self, request, context):
        """Method to create longrunning recognition operation. Created operation will persist for a limited time and will be deleted after that time has expired.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_SpeechToTextServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'Recognize': grpc.unary_unary_rpc_method_handler(
                    servicer.Recognize,
                    request_deserializer=tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.RecognizeRequest.FromString,
                    response_serializer=tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.RecognizeResponse.SerializeToString,
            ),
            'StreamingRecognize': grpc.stream_stream_rpc_method_handler(
                    servicer.StreamingRecognize,
                    request_deserializer=tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.StreamingRecognizeRequest.FromString,
                    response_serializer=tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.StreamingRecognizeResponse.SerializeToString,
            ),
            'LongRunningRecognize': grpc.unary_unary_rpc_method_handler(
                    servicer.LongRunningRecognize,
                    request_deserializer=tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.LongRunningRecognizeRequest.FromString,
                    response_serializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.Operation.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'tinkoff.cloud.stt.v1.SpeechToText', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class SpeechToText(object):
    """Speech recognition.
    """

    @staticmethod
    def Recognize(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/tinkoff.cloud.stt.v1.SpeechToText/Recognize',
            tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.RecognizeRequest.SerializeToString,
            tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.RecognizeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamingRecognize(request_iterator,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.stream_stream(request_iterator, target, '/tinkoff.cloud.stt.v1.SpeechToText/StreamingRecognize',
            tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.StreamingRecognizeRequest.SerializeToString,
            tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.StreamingRecognizeResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def LongRunningRecognize(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/tinkoff.cloud.stt.v1.SpeechToText/LongRunningRecognize',
            tinkoff_dot_cloud_dot_stt_dot_v1_dot_stt__pb2.LongRunningRecognizeRequest.SerializeToString,
            tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from astersay.backends.tinkoff.proto import tts_pb2 as tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2


class TextToSpeechStub(object):
    """Speech synthesis.
    """

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.ListVoices = channel.unary_unary(
                '/tinkoff.cloud.tts.v1.TextToSpeech/ListVoices',
                request_serializer=tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.ListVoicesRequest.SerializeToString,
                response_deserializer=tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.ListVoicesResponses.FromString,
                )
        self.Synthesize = channel.unary_unary(
                '/tinkoff.cloud.tts.v1.TextToSpeech/Synthesize',
                request_serializer=tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.SynthesizeSpeechRequest.SerializeToString,
                response_deserializer=tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.SynthesizeSpeechResponse.FromString,
                )
        self.StreamingSynthesize = channel.unary_stream(
                '/tinkoff.cloud.tts.v1.TextToSpeech/StreamingSynthesize',
                request_serializer=tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.SynthesizeSpeechRequest.SerializeToString,
                response_deserializer=tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.StreamingSynthesizeSpeechResponse.FromString,
                )


class TextToSpeechServicer(object):
    """Speech synthesis.
    """

    def ListVoices(self, request, context):
        """<i>Not implemented</i> Method for retrieving available voice list.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def Synthesize(self, request, context):
        """<i>Not implemented</i> Method for fragment synthesis.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def StreamingSynthesize(self, request, context):
        """Method for streaming synthesis.
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_TextToSpeechServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'ListVoices': grpc.unary_unary_rpc_method_handler(
                    servicer.ListVoices,
                    request_deserializer=tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.ListVoicesRequest.FromString,
                    response_serializer=tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.ListVoicesResponses.SerializeToString,
            ),
            'Synthesize': grpc.unary_unary_rpc_method_handler(
                    servicer.Synthesize,
                    request_deserializer=tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.SynthesizeSpeechRequest.FromString,
                    response_serializer=tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.SynthesizeSpeechResponse.SerializeToString,
            ),
            'StreamingSynthesize': grpc.unary_stream_rpc_method_handler(
                    servicer.StreamingSynthesize,
                    request_deserializer=tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.SynthesizeSpeechRequest.FromString,
                    response_serializer=tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.StreamingSynthesizeSpeechResponse.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'tinkoff.cloud.tts.v1.TextToSpeech', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class TextToSpeech(object):
    """Speech synthesis.
    """

    @staticmethod
    def ListVoices(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/tinkoff.cloud.tts.v1.TextToSpeech/ListVoices',
            tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.ListVoicesRequest.SerializeToString,
            tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.ListVoicesResponses.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def Synthesize(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/tinkoff.cloud.tts.v1.TextToSpeech/Synthesize',
            tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.SynthesizeSpeechRequest.SerializeToString,
            tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.SynthesizeSpeechResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def StreamingSynthesize(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/tinkoff.cloud.tts.v1.TextToSpeech/StreamingSynthesize',
            tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.SynthesizeSpeechRequest.SerializeToString,
            tinkoff_dot_cloud_dot_tts_dot_v1_dot_tts__pb2.StreamingSynthesizeSpeechResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



# Generated by the gRPC Python protocol compiler plugin. DO NOT EDIT!
"""Client and server classes corresponding to protobuf-defined services."""
import grpc

from google.protobuf import empty_pb2 as google_dot_protobuf_dot_empty__pb2
from astersay.backends.tinkoff.proto import longrunning_pb2 as tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2


class OperationsStub(object):
    """Missing associated documentation comment in .proto file."""

    def __init__(self, channel):
        """Constructor.

        Args:
            channel: A grpc.Channel.
        """
        self.GetOperation = channel.unary_unary(
                '/tinkoff.cloud.longrunning.v1.Operations/GetOperation',
                request_serializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.GetOperationRequest.SerializeToString,
                response_deserializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.Operation.FromString,
                )
        self.WaitOperation = channel.unary_unary(
                '/tinkoff.cloud.longrunning.v1.Operations/WaitOperation',
                request_serializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.WaitOperationRequest.SerializeToString,
                response_deserializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.Operation.FromString,
                )
        self.ListOperations = channel.unary_unary(
                '/tinkoff.cloud.longrunning.v1.Operations/ListOperations',
                request_serializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.ListOperationsRequest.SerializeToString,
                response_deserializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.ListOperationsResponse.FromString,
                )
        self.WatchOperations = channel.unary_stream(
                '/tinkoff.cloud.longrunning.v1.Operations/WatchOperations',
                request_serializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.WatchOperationsRequest.SerializeToString,
                response_deserializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.WatchOperationsResponse.FromString,
                )
        self.DeleteOperation = channel.unary_unary(
                '/tinkoff.cloud.longrunning.v1.Operations/DeleteOperation',
                request_serializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.DeleteOperationRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )
        self.CancelOperation = channel.unary_unary(
                '/tinkoff.cloud.longrunning.v1.Operations/CancelOperation',
                request_serializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.CancelOperationRequest.SerializeToString,
                response_deserializer=google_dot_protobuf_dot_empty__pb2.Empty.FromString,
                )


class OperationsServicer(object):
    """Missing associated documentation comment in .proto file."""

    def GetOperation(self, request, context):
        """Starts polling for operation statuses
        Returns operation status
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WaitOperation(self, request, context):
        """Wait for operation update
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def ListOperations(self, request, context):
        """List operations
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def WatchOperations(self, request, context):
        """Watch operations
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def DeleteOperation(self, request, context):
        """Deletes spepcified operations
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')

    def CancelOperation(self, request, context):
        """Cancels spepcified operations
        """
        context.set_code(grpc.StatusCode.UNIMPLEMENTED)
        context.set_details('Method not implemented!')
        raise NotImplementedError('Method not implemented!')


def add_OperationsServicer_to_server(servicer, server):
    rpc_method_handlers = {
            'GetOperation': grpc.unary_unary_rpc_method_handler(
                    servicer.GetOperation,
                    request_deserializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.GetOperationRequest.FromString,
                    response_serializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.Operation.SerializeToString,
            ),
            'WaitOperation': grpc.unary_unary_rpc_method_handler(
                    servicer.WaitOperation,
                    request_deserializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.WaitOperationRequest.FromString,
                    response_serializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.Operation.SerializeToString,
            ),
            'ListOperations': grpc.unary_unary_rpc_method_handler(
                    servicer.ListOperations,
                    request_deserializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.ListOperationsRequest.FromString,
                    response_serializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.ListOperationsResponse.SerializeToString,
            ),
            'WatchOperations': grpc.unary_stream_rpc_method_handler(
                    servicer.WatchOperations,
                    request_deserializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.WatchOperationsRequest.FromString,
                    response_serializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.WatchOperationsResponse.SerializeToString,
            ),
            'DeleteOperation': grpc.unary_unary_rpc_method_handler(
                    servicer.DeleteOperation,
                    request_deserializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.DeleteOperationRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
            'CancelOperation': grpc.unary_unary_rpc_method_handler(
                    servicer.CancelOperation,
                    request_deserializer=tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.CancelOperationRequest.FromString,
                    response_serializer=google_dot_protobuf_dot_empty__pb2.Empty.SerializeToString,
            ),
    }
    generic_handler = grpc.method_handlers_generic_handler(
            'tinkoff.cloud.longrunning.v1.Operations', rpc_method_handlers)
    server.add_generic_rpc_handlers((generic_handler,))


 # This class is part of an EXPERIMENTAL API.
class Operations(object):
    """Missing associated documentation comment in .proto file."""

    @staticmethod
    def GetOperation(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/tinkoff.cloud.longrunning.v1.Operations/GetOperation',
            tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.GetOperationRequest.SerializeToString,
            tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WaitOperation(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/tinkoff.cloud.longrunning.v1.Operations/WaitOperation',
            tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.WaitOperationRequest.SerializeToString,
            tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.Operation.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def ListOperations(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/tinkoff.cloud.longrunning.v1.Operations/ListOperations',
            tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.ListOperationsRequest.SerializeToString,
            tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.ListOperationsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def WatchOperations(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_stream(request, target, '/tinkoff.cloud.longrunning.v1.Operations/WatchOperations',
            tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.WatchOperationsRequest.SerializeToString,
            tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.WatchOperationsResponse.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def DeleteOperation(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/tinkoff.cloud.longrunning.v1.Operations/DeleteOperation',
            tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.DeleteOperationRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)

    @staticmethod
    def CancelOperation(request,
            target,
            options=(),
            channel_credentials=None,
            call_credentials=None,
            insecure=False,
            compression=None,
            wait_for_ready=None,
            timeout=None,
            metadata=None):
        return grpc.experimental.unary_unary(request, target, '/tinkoff.cloud.longrunning.v1.Operations/CancelOperation',
            tinkoff_dot_cloud_dot_longrunning_dot_v1_dot_longrunning__pb2.CancelOperationRequest.SerializeToString,
            google_dot_protobuf_dot_empty__pb2.Empty.FromString,
            options, channel_credentials,
            insecure, call_credentials, compression, wait_for_ready, timeout, metadata)



#
# Copyright (c) 2020, Grigoriy Kramarenko
# All rights reserved.
# This file is distributed under the same license as the current project.
#
import re
import pymorphy2


words_pattern = re.compile(r'[\w\-]+')


def words_iterator(text):
    for w in words_pattern.findall(text):
        yield w


class Pymorphy2Processor:
    parse = pymorphy2.MorphAnalyzer().parse

    # Имена, которые не определяет pymorphy2.
    extra_names = (
        'Роман',
    )

    def parse_names(self, text):
        """Метод ищет в тексте все имена."""
        parse = self.parse
        # probability score threshold
        prob_thresh = 0.4
        L = []
        extra = self.extra_names
        for word in words_iterator(text):
            for p in parse(word):
                if 'Name' in p.tag and p.score >= prob_thresh:
                    L.append(p.normal_form.title())
                    break
                elif 'Patr' in p.tag and p.score >= prob_thresh:
                    L.append(word.title())
                    break
                elif word in extra:
                    L.append(word.title())
                    break
        return L



#
#  Copyright (c) 2021. Egor Demushin.
#  All rights reserved.
#  This file is distributed under the same license as the current project.
#
from astersay.backends.base import BaseBackend
from astersay.backends.tinkoff.token import TinkoffTokenManager


class TinkoffBaseBackend(BaseBackend):
    token_manager = None

    def set_token_manager(self, settings, force=False):
        """
        Устанавливает из любого экземпляра общий для основного класса менеджер
        токена.
        """
        if self.token_manager and not force:
            return self.token_manager
        tm = TinkoffTokenManager(**settings.tinkoff['auth'])
        TinkoffBaseBackend.token_manager = tm
        return tm
